[gold lto patch] Add plugin support for LDPR_PREVAILING_DEF_IRONLY_EXP

Cary Coutant ccoutant@google.com
Mon Sep 26 20:39:00 GMT 2011


This patch adds LDPT_GET_SYMBOLS_V2 to the gold plugin API, which will
return the new LDPR_PREVAILING_DEF_IRONLY_EXP resolution code for
symbols that are referenced only from IR code, but are exported. For
more details, see GCC PR lto/47247:

   http://gcc.gnu.org/bugzilla/show_bug.cgi?id=47247

Tested on x86_64. OK to commit?

-cary

PS: Is this the right way to put GCC PR tags in a binutils ChangeLog?

PPS: Sorry, Honza, for taking so long. I had the implementation done
at the London conference, but never found the time to update the test
suite.


include/ChangeLog

	gcc PR lto/47247
	* plugin-api.h (enum ld_plugin_symbol_resolution): Add
	LDPR_PREVAILING_DEF_IRONLY_EXP.
	(enum ld_plugin_tag): Add LDPT_GET_SYMBOLS_V2.

gold/ChangeLog

	gcc PR lto/47247
	* plugin.cc (get_symbols_v2): New function.
	(Plugin::load): Add LDPT_GET_SYMBOLS_V2.
	(is_referenced_from_outside): New function.
	(Pluginobj::get_symbol_resolution_info): Add version parameter, return
	LDPR_PREVAILING_DEF_IRONLY_EXP when using new version.
	(get_symbols): Pass version parameter.
	(get_symbols_v2): New function.
	* plugin.h (Pluginobj::get_symbol_resolution_info): Add version
	parameter.
	* testsuite/plugin_test.c (get_symbols_v2): New static variable.
	(onload): Add LDPT_GET_SYMBOLS_V2.
	(all_symbols_read_hook): Use get_symbols_v2; check for
	LDPR_PREVAILING_DEF_IRONLY_EXP.
	* testsuite/plugin_test_3.sh: Update expected results.
-------------- next part --------------
2011-09-26  Cary Coutant  <ccoutant@google.com>

include/ChangeLog

	* plugin-api.h (enum ld_plugin_symbol_resolution): Add
	LDPR_PREVAILING_DEF_IRONLY_EXP.
	(enum ld_plugin_tag): Add LDPT_GET_SYMBOLS_V2.

gold/ChangeLog

	* plugin.cc (get_symbols_v2): New function.
	(Plugin::load): Add LDPT_GET_SYMBOLS_V2.
	(is_referenced_from_outside): New function.
	(Pluginobj::get_symbol_resolution_info): Add version parameter, return
	LDPR_PREVAILING_DEF_IRONLY_EXP when using new version.
	(get_symbols): Pass version parameter.
	(get_symbols_v2): New function.
	* plugin.h (Pluginobj::get_symbol_resolution_info): Add version
	parameter.
	* testsuite/plugin_test.c (get_symbols_v2): New static variable.
	(onload): Add LDPT_GET_SYMBOLS_V2.
	(all_symbols_read_hook): Use get_symbols_v2; check for
	LDPR_PREVAILING_DEF_IRONLY_EXP.
	* testsuite/plugin_test_3.sh: Update expected results.


commit 0948cfa152ae1fc41285941ddbd8176af9e19387
Author: Cary Coutant <ccoutant@google.com>
Date:   Thu Jun 23 10:25:33 2011 -0700

    Update plugin API with get_symbols_v2.

diff --git a/gold/plugin.h b/gold/plugin.h
index 4c9445a..32ffe35 100644
--- a/gold/plugin.h
+++ b/gold/plugin.h
@@ -393,7 +393,9 @@ class Pluginobj : public Object
 
   // Fill in the symbol resolution status for the given plugin symbols.
   ld_plugin_status
-  get_symbol_resolution_info(int nsyms, ld_plugin_symbol* syms) const;
+  get_symbol_resolution_info(int nsyms,
+			     ld_plugin_symbol* syms,
+			     int version) const;
 
   // Store the incoming symbols from the plugin for later processing.
   void
diff --git a/gold/testsuite/plugin_test.c b/gold/testsuite/plugin_test.c
index 619a160..47d400a 100644
--- a/gold/testsuite/plugin_test.c
+++ b/gold/testsuite/plugin_test.c
@@ -56,6 +56,7 @@ static ld_plugin_register_all_symbols_read register_all_symbols_read_hook = NULL
 static ld_plugin_register_cleanup register_cleanup_hook = NULL;
 static ld_plugin_add_symbols add_symbols = NULL;
 static ld_plugin_get_symbols get_symbols = NULL;
+static ld_plugin_get_symbols get_symbols_v2 = NULL;
 static ld_plugin_add_input_file add_input_file = NULL;
 static ld_plugin_message message = NULL;
 static ld_plugin_get_input_file get_input_file = NULL;
@@ -120,6 +121,9 @@ onload(struct ld_plugin_tv *tv)
         case LDPT_GET_SYMBOLS:
           get_symbols = entry->tv_u.tv_get_symbols;
           break;
+        case LDPT_GET_SYMBOLS_V2:
+          get_symbols_v2 = entry->tv_u.tv_get_symbols;
+          break;
         case LDPT_ADD_INPUT_FILE:
           add_input_file = entry->tv_u.tv_add_input_file;
           break;
@@ -394,9 +398,9 @@ all_symbols_read_hook(void)
 
   (*message)(LDPL_INFO, "all symbols read hook called");
 
-  if (get_symbols == NULL)
+  if (get_symbols_v2 == NULL)
     {
-      fprintf(stderr, "tv_get_symbols interface missing\n");
+      fprintf(stderr, "tv_get_symbols (v2) interface missing\n");
       return LDPS_ERR;
     }
 
@@ -404,7 +408,7 @@ all_symbols_read_hook(void)
        claimed_file != NULL;
        claimed_file = claimed_file->next)
     {
-      (*get_symbols)(claimed_file->handle, claimed_file->nsyms,
+      (*get_symbols_v2)(claimed_file->handle, claimed_file->nsyms,
                      claimed_file->syms);
 
       for (i = 0; i < claimed_file->nsyms; ++i)
@@ -423,6 +427,9 @@ all_symbols_read_hook(void)
             case LDPR_PREVAILING_DEF_IRONLY:
               res = "PREVAILING_DEF_IRONLY";
               break;
+            case LDPR_PREVAILING_DEF_IRONLY_EXP:
+              res = "PREVAILING_DEF_IRONLY_EXP";
+              break;
             case LDPR_PREEMPTED_REG:
               res = "PREEMPTED_REG";
               break;
diff --git a/gold/testsuite/plugin_test_3.sh b/gold/testsuite/plugin_test_3.sh
index 961df15..39356d1 100755
--- a/gold/testsuite/plugin_test_3.sh
+++ b/gold/testsuite/plugin_test_3.sh
@@ -46,7 +46,7 @@ check plugin_test_3.err "two_file_test_main.o: claim file hook called"
 check plugin_test_3.err "two_file_test_1.syms: claim file hook called"
 check plugin_test_3.err "two_file_test_1b.syms: claim file hook called"
 check plugin_test_3.err "two_file_test_2.syms: claim file hook called"
-check plugin_test_3.err "two_file_test_1.syms: _Z4f13iv: PREVAILING_DEF_REG"
+check plugin_test_3.err "two_file_test_1.syms: _Z4f13iv: PREVAILING_DEF_IRONLY_EXP"
 check plugin_test_3.err "two_file_test_1.syms: _Z2t2v: PREVAILING_DEF_REG"
 check plugin_test_3.err "two_file_test_1.syms: v2: RESOLVED_IR"
 check plugin_test_3.err "two_file_test_1.syms: t17data: RESOLVED_IR"
diff --git a/include/plugin-api.h b/include/plugin-api.h
index df00393..122424c 100644
--- a/include/plugin-api.h
+++ b/include/plugin-api.h
@@ -155,7 +155,13 @@ enum ld_plugin_symbol_resolution
   LDPR_RESOLVED_EXEC,
 
   /* This symbol was resolved by a definition in a shared object.  */
-  LDPR_RESOLVED_DYN
+  LDPR_RESOLVED_DYN,
+
+  /* This is the prevailing definition of the symbol, with no
+     references from regular objects.  It is only referenced from IR
+     code, but the symbol is exported and may be referenced from
+     a dynamic object (not seen at link time).  */
+  LDPR_PREVAILING_DEF_IRONLY_EXP
 };
 
 /* The plugin library's "claim file" handler.  */
@@ -347,7 +353,8 @@ enum ld_plugin_tag
   LDPT_GET_INPUT_SECTION_NAME,
   LDPT_GET_INPUT_SECTION_CONTENTS,
   LDPT_UPDATE_SECTION_ORDER,
-  LDPT_ALLOW_SECTION_ORDERING
+  LDPT_ALLOW_SECTION_ORDERING,
+  LDPT_GET_SYMBOLS_V2
 };
 
 /* The plugin transfer vector.  */


More information about the Binutils mailing list
diff --git a/gold/plugin.cc b/gold/plugin.cc
index 8f6e87f..3ccd8d0 100644
--- a/gold/plugin.cc
+++ b/gold/plugin.cc
@@ -78,6 +78,9 @@ static enum ld_plugin_status
 get_symbols(const void *handle, int nsyms, struct ld_plugin_symbol *syms);
 
 static enum ld_plugin_status
+get_symbols_v2(const void *handle, int nsyms, struct ld_plugin_symbol *syms);
+
+static enum ld_plugin_status
 add_input_file(const char *pathname);
 
 static enum ld_plugin_status
@@ -156,7 +159,7 @@ Plugin::load()
   sscanf(ver, "%d.%d", &major, &minor);
 
   // Allocate and populate a transfer vector.
-  const int tv_fixed_size = 23;
+  const int tv_fixed_size = 24;
 
   int tv_size = this->args_.size() + tv_fixed_size;
   ld_plugin_tv* tv = new ld_plugin_tv[tv_size];
@@ -228,6 +231,10 @@ Plugin::load()
   tv[i].tv_u.tv_get_symbols = get_symbols;
 
   ++i;
+  tv[i].tv_tag = LDPT_GET_SYMBOLS_V2;
+  tv[i].tv_u.tv_get_symbols = get_symbols_v2;
+
+  ++i;
   tv[i].tv_tag = LDPT_ADD_INPUT_FILE;
   tv[i].tv_u.tv_add_input_file = add_input_file;
 
@@ -810,11 +817,11 @@ Pluginobj::Pluginobj(const std::string& name, Input_file* input_file,
 {
 }
 
 // Return TRUE if a defined symbol might be reachable from outside the
 // universe of claimed objects.
 
 static inline bool
-is_visible_from_outside(Symbol* lsym)
+is_referenced_from_outside(Symbol* lsym)
 {
   if (lsym->in_real_elf())
     return true;
@@ -822,6 +829,15 @@ is_visible_from_outside(Symbol* lsym)
     return true;
   if (parameters->options().is_undefined(lsym->name()))
     return true;
+  return false;
+}
+
+// Return TRUE if a defined symbol might be reachable from outside the
+// load module.
+
+static inline bool
+is_visible_from_outside(Symbol* lsym)
+{
   if (parameters->options().export_dynamic() || parameters->options().shared())
     return lsym->is_externally_visible();
   return false;
@@ -830,8 +846,18 @@ is_visible_from_outside(Symbol* lsym)
 // Get symbol resolution info.
 
 ld_plugin_status
-Pluginobj::get_symbol_resolution_info(int nsyms, ld_plugin_symbol* syms) const
-{
+Pluginobj::get_symbol_resolution_info(int nsyms,
+				      ld_plugin_symbol* syms,
+				      int version) const
+{
+  // For version 1 of this interface, we cannot use
+  // LDPR_PREVAILING_DEF_IRONLY_EXP, so we return LDPR_PREVAILING_DEF
+  // instead.
+  const ld_plugin_symbol_resolution ldpr_prevailing_def_ironly_exp
+      = (version > 1
+	 ? LDPR_PREVAILING_DEF_IRONLY_EXP
+	 : LDPR_PREVAILING_DEF);
+
   if (nsyms > this->nsyms_)
     return LDPS_NO_SYMS;
 
@@ -862,9 +888,14 @@ Pluginobj::get_symbol_resolution_info(int nsyms, ld_plugin_symbol* syms) const
           if (lsym->source() != Symbol::FROM_OBJECT)
             res = LDPR_RESOLVED_EXEC;
           else if (lsym->object()->pluginobj() == this)
-            res = (is_visible_from_outside(lsym)
-                   ? LDPR_PREVAILING_DEF
-                   : LDPR_PREVAILING_DEF_IRONLY);
+	    {
+	      if (is_referenced_from_outside(lsym))
+		res = LDPR_PREVAILING_DEF;
+	      else if (is_visible_from_outside(lsym))
+		res = ldpr_prevailing_def_ironly_exp;
+	      else
+		res = LDPR_PREVAILING_DEF_IRONLY;
+	    }
           else if (lsym->object()->pluginobj() != NULL)
             res = LDPR_RESOLVED_IR;
           else if (lsym->object()->is_dynamic())
@@ -878,9 +909,14 @@ Pluginobj::get_symbol_resolution_info(int nsyms, ld_plugin_symbol* syms) const
           if (lsym->source() != Symbol::FROM_OBJECT)
             res = LDPR_PREEMPTED_REG;
           else if (lsym->object() == static_cast<const Object*>(this))
-            res = (is_visible_from_outside(lsym)
-                   ? LDPR_PREVAILING_DEF
-                   : LDPR_PREVAILING_DEF_IRONLY);
+	    {
+	      if (is_referenced_from_outside(lsym))
+		res = LDPR_PREVAILING_DEF;
+	      else if (is_visible_from_outside(lsym))
+		res = ldpr_prevailing_def_ironly_exp;
+	      else
+		res = LDPR_PREVAILING_DEF_IRONLY;
+	    }
           else
             res = (lsym->object()->pluginobj() != NULL
                    ? LDPR_PREEMPTED_IR
@@ -1411,7 +1447,24 @@ get_symbols(const void* handle, int nsyms, ld_plugin_symbol* syms)
   Pluginobj* plugin_obj = obj->pluginobj();
   if (plugin_obj == NULL)
     return LDPS_ERR;
-  return plugin_obj->get_symbol_resolution_info(nsyms, syms);
+  return plugin_obj->get_symbol_resolution_info(nsyms, syms, 1);
+}
+
+// Version 2 of the above.  The only difference is that this version
+// is allowed to return the resolution code LDPR_PREVAILING_DEF_IRONLY_EXP.
+
+static enum ld_plugin_status
+get_symbols_v2(const void* handle, int nsyms, ld_plugin_symbol* syms)
+{
+  gold_assert(parameters->options().has_plugins());
+  Object* obj = parameters->options().plugins()->object(
+    static_cast<unsigned int>(reinterpret_cast<intptr_t>(handle)));
+  if (obj == NULL)
+    return LDPS_ERR;
+  Pluginobj* plugin_obj = obj->pluginobj();
+  if (plugin_obj == NULL)
+    return LDPS_ERR;
+  return plugin_obj->get_symbol_resolution_info(nsyms, syms, 2);
 }
 
 // Add a new (real) input file generated by a plugin.
@@ -810,11 +817,11 @@ Pluginobj::Pluginobj(const std::string& name, Input_file* input_file,
 }
 
-// Return TRUE if a defined symbol might be reachable from outside the
+// Return TRUE if a defined symbol is referenced from outside the
 // universe of claimed objects.
 